aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
-rw-r--r--gdb/ChangeLog9
-rw-r--r--gdb/dwarf2read.c94
-rw-r--r--gdb/testsuite/ChangeLog6
-rw-r--r--gdb/testsuite/gdb.dwarf2/pr13961.S355
-rw-r--r--gdb/testsuite/gdb.dwarf2/pr13961.exp36
5 files changed, 431 insertions, 69 deletions
diff --git a/gdb/ChangeLog b/gdb/ChangeLog
index 23e6e30..8c57df6 100644
--- a/gdb/ChangeLog
+++ b/gdb/ChangeLog
@@ -1,3 +1,12 @@
+2012-04-30 Doug Evans <dje@google.com>
+
+ * dwarf2read.c (init_cutu_and_read_dies): Renamed from
+ init_and_read_dies_worker. All callers updated.
+ (init_cu_and_read_dies, init_tu_and_read_dies): Delete. All calls
+ replaced with init_cutu_and_read_dies.
+ (load_partial_comp_unit): Pass 1 for use_existing_cu.
+ (find_partial_die): Remove FIXME. Don't free current CU.
+
2012-04-30 Sterling Augustine <saugustine@google.com>
* contrib: New directory.
diff --git a/gdb/testsuite/ChangeLog b/gdb/testsuite/ChangeLog
index ce820bc..074beff 100644
--- a/gdb/testsuite/ChangeLog
+++ b/gdb/testsuite/ChangeLog
@@ -1,3 +1,9 @@
+2012-04-30 Doug Evans <dje@google.com>
+
+ PR testsuite/13961
+ * gdb.dwarf2/pr13961.S: New file.
+ * gdb.dwarf2/pr13961.exp: New file.
+
2012-04-29 Yao Qi <yao@codesourcery.com>
* gdb.dwarf2/dw2-ifort-parameter.exp: Use proc `dwarf2_support'.
diff --git a/gdb/testsuite/gdb.dwarf2/pr13961.S b/gdb/testsuite/gdb.dwarf2/pr13961.S
new file mode 100644
index 0000000..8d333e7
--- /dev/null
+++ b/gdb/testsuite/gdb.dwarf2/pr13961.S
@@ -0,0 +1,355 @@
+/* Copyright 2012 Free Software Foundation, Inc.
+
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 3 of the License, or
+ (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with this program. If not, see <http://www.gnu.org/licenses/>. */
+
+/* Testcase to exercise the code path in bug 13961.
+
+ Compiled from:
+
+ class foo
+ {
+ public:
+ int bar;
+ };
+
+ foo baz;
+
+ int
+ main ()
+ {
+ return 0;
+ }
+
+ And then manually edited to insert the necessary DW_AT_specification
+ entries to trigger the desired code path.
+ There's no real need to make this architecture-specific, so it has been
+ further hand-edited to support that.
+*/
+
+ .file "pr13961.cc"
+
+ .globl baz
+ .bss
+ .align 4
+ .type baz, @object
+ .size baz, 4
+baz:
+ .zero 4
+
+ .text
+.Ltext0:
+ .globl main
+ .type main, @function
+main:
+.LFB0:
+ .4byte 0
+.LFE0:
+ .size main, .-main
+.Letext0:
+
+ .section .debug_types,"",@progbits
+.Ldebug_types0:
+ .4byte .Ltu_end - .Ltu_start /* Length of Compilation Unit Info */
+.Ltu_start:
+ .2byte 0x4 /* DWARF version number */
+ .4byte .Ldebug_abbrev0 /* Offset Into Abbrev. Section */
+ .byte 0x4 /* Pointer Size (in bytes) */
+ .byte 0x19 /* Type Signature */
+ .byte 0x8b
+ .byte 0x7e
+ .byte 0xac
+ .byte 0xce
+ .byte 0xf1
+ .byte 0x22
+ .byte 0x90
+ .4byte .Ltu_class_type - .Ldebug_types0 /* Offset to Type DIE */
+ .uleb128 0x1 /* (DIE (0x17) DW_TAG_type_unit) */
+ .byte 0x4 /* DW_AT_language */
+ .byte 0x73 /* DW_AT_GNU_odr_signature */
+ .byte 0xea
+ .byte 0x85
+ .byte 0x23
+ .byte 0x75
+ .byte 0x8a
+ .byte 0x7e
+ .byte 0x87
+ .4byte .Ldebug_line0 /* DW_AT_stmt_list */
+
+ /* Manually inserted to have a DW_AT_specification refering to
+ something and appearing ahead of it. */
+ .uleb128 0x8 /* DW_TAG_class_type */
+ .4byte .Ltu_class_type - .Ldebug_types0
+ /* End of manual insertion */
+
+.Ltu_class_type:
+ .uleb128 0x2 /* (DIE (0x25) DW_TAG_class_type) */
+ .ascii "foo\0" /* DW_AT_name */
+ .byte 0x4 /* DW_AT_byte_size */
+ .byte 0x1 /* DW_AT_decl_file (pr13961.cc) */
+ .byte 0x1 /* DW_AT_decl_line */
+ .4byte 0x3f /* DW_AT_sibling */
+ .uleb128 0x3 /* (DIE (0x31) DW_TAG_member) */
+ .ascii "bar\0" /* DW_AT_name */
+ .byte 0x1 /* DW_AT_decl_file (pr13961.cc) */
+ .byte 0x4 /* DW_AT_decl_line */
+ .4byte 0x3f /* DW_AT_type */
+ .byte 0 /* DW_AT_data_member_location */
+ .byte 0x1 /* DW_AT_accessibility */
+ .byte 0 /* end of children of DIE 0x25 */
+ .uleb128 0x4 /* (DIE (0x3f) DW_TAG_base_type) */
+ .byte 0x4 /* DW_AT_byte_size */
+ .byte 0x5 /* DW_AT_encoding */
+ .ascii "int\0" /* DW_AT_name */
+ .byte 0 /* end of children of DIE 0x17 */
+
+.Ltu_end:
+
+ .section .debug_info,"",@progbits
+.Ldebug_info0:
+ .4byte .Lcu_end - .Lcu_start /* Length of Compilation Unit Info */
+.Lcu_start:
+ .2byte 0x4 /* DWARF version number */
+ .4byte .Ldebug_abbrev0 /* Offset Into Abbrev. Section */
+ .byte 0x4 /* Pointer Size (in bytes) */
+ .uleb128 0x5 /* (DIE (0xb) DW_TAG_compile_unit) */
+ .4byte .LASF0 /* DW_AT_producer: "GNU C++ 4.6.3 20120306" */
+ .byte 0x4 /* DW_AT_language */
+ .4byte .LASF1 /* DW_AT_name: "pr13961.cc" */
+ .4byte .LASF2 /* DW_AT_comp_dir: "." */
+ .4byte .Ltext0 /* DW_AT_low_pc */
+ .4byte .Letext0 /* DW_AT_high_pc */
+ .4byte .Ldebug_line0 /* DW_AT_stmt_list */
+.Lint_base_type:
+ .uleb128 0x4 /* (DIE (0x25) DW_TAG_base_type) */
+ .byte 0x4 /* DW_AT_byte_size */
+ .byte 0x5 /* DW_AT_encoding */
+ .ascii "int\0" /* DW_AT_name */
+ .uleb128 0x6 /* (DIE (0x2c) DW_TAG_subprogram) */
+ /* DW_AT_external */
+ .4byte .LASF3 /* DW_AT_name: "main" */
+ .byte 0x1 /* DW_AT_decl_file (pr13961.cc) */
+ .byte 0xa /* DW_AT_decl_line */
+ .4byte 0x25 /* DW_AT_type */
+ .4byte .LFB0 /* DW_AT_low_pc */
+ .4byte .LFE0 /* DW_AT_high_pc */
+ .uleb128 0x1 /* DW_AT_frame_base */
+ .byte 0x9c /* DW_OP_call_frame_cfa */
+ /* DW_AT_GNU_all_call_sites */
+ .uleb128 0x7 /* (DIE (0x41) DW_TAG_variable) */
+ .ascii "baz\0" /* DW_AT_name */
+ .byte 0x1 /* DW_AT_decl_file (pr13961.cc) */
+ .byte 0x7 /* DW_AT_decl_line */
+ .byte 0x19 /* DW_AT_type */
+ .byte 0x8b
+ .byte 0x7e
+ .byte 0xac
+ .byte 0xce
+ .byte 0xf1
+ .byte 0x22
+ .byte 0x90
+ /* DW_AT_external */
+ .uleb128 0x5 /* DW_AT_location */
+ .byte 0x3 /* DW_OP_addr */
+ .4byte baz
+
+ /* Manually inserted to have a DW_AT_specification refering to
+ something and appearing ahead of it. */
+ .uleb128 0x8 /* DW_TAG_class_type */
+ .4byte .Lcu_class_type - .Ldebug_info0 /* DW_AT_specification */
+
+.Lcu_int_type:
+ .uleb128 0x4 /* (DIE (0x3f) DW_TAG_base_type) */
+ .byte 0x4 /* DW_AT_byte_size */
+ .byte 0x5 /* DW_AT_encoding */
+ .ascii "int\0" /* DW_AT_name */
+.Lcu_class_type:
+ .uleb128 0x2 /* (DIE (0x25) DW_TAG_class_type) */
+ .ascii "foo2\0" /* DW_AT_name */
+ .byte 0x4 /* DW_AT_byte_size */
+ .byte 0x1 /* DW_AT_decl_file (pr13961.cc) */
+ .byte 0x1 /* DW_AT_decl_line */
+ .4byte 0x3f /* DW_AT_sibling */
+ .uleb128 0x3 /* (DIE (0x31) DW_TAG_member) */
+ .ascii "bar\0" /* DW_AT_name */
+ .byte 0x1 /* DW_AT_decl_file (pr13961.cc) */
+ .byte 0x4 /* DW_AT_decl_line */
+ .4byte .Lcu_int_type - .Ldebug_info0 /* DW_AT_type */
+ .byte 0 /* DW_AT_data_member_location */
+ .byte 0x1 /* DW_AT_accessibility */
+ .byte 0 /* end of children of DIE 0x25 */
+ .uleb128 0x4 /* (DIE (0x3f) DW_TAG_base_type) */
+ .byte 0x4 /* DW_AT_byte_size */
+ .byte 0x5 /* DW_AT_encoding */
+ .ascii "int\0" /* DW_AT_name */
+ .byte 0 /* end of children of DIE 0x17 */
+ /* End of manual insertion */
+
+ .byte 0 /* end of children of DIE 0xb */
+
+.Lcu_end:
+
+ .section .debug_abbrev,"",@progbits
+.Ldebug_abbrev0:
+ .uleb128 0x1 /* (abbrev code) */
+ .uleb128 0x41 /* (TAG: DW_TAG_type_unit) */
+ .byte 0x1 /* DW_children_yes */
+ .uleb128 0x13 /* (DW_AT_language) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x210f /* (DW_AT_GNU_odr_signature) */
+ .uleb128 0x7 /* (DW_FORM_data8) */
+ .uleb128 0x10 /* (DW_AT_stmt_list) */
+ .uleb128 0x17 /* (DW_FORM_sec_offset) */
+ .byte 0
+ .byte 0
+ .uleb128 0x2 /* (abbrev code) */
+ .uleb128 0x2 /* (TAG: DW_TAG_class_type) */
+ .byte 0x1 /* DW_children_yes */
+ .uleb128 0x3 /* (DW_AT_name) */
+ .uleb128 0x8 /* (DW_FORM_string) */
+ .uleb128 0xb /* (DW_AT_byte_size) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3a /* (DW_AT_decl_file) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3b /* (DW_AT_decl_line) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x1 /* (DW_AT_sibling) */
+ .uleb128 0x13 /* (DW_FORM_ref4) */
+ .byte 0
+ .byte 0
+ .uleb128 0x3 /* (abbrev code) */
+ .uleb128 0xd /* (TAG: DW_TAG_member) */
+ .byte 0 /* DW_children_no */
+ .uleb128 0x3 /* (DW_AT_name) */
+ .uleb128 0x8 /* (DW_FORM_string) */
+ .uleb128 0x3a /* (DW_AT_decl_file) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3b /* (DW_AT_decl_line) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x49 /* (DW_AT_type) */
+ .uleb128 0x13 /* (DW_FORM_ref4) */
+ .uleb128 0x38 /* (DW_AT_data_member_location) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x32 /* (DW_AT_accessibility) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .byte 0
+ .byte 0
+ .uleb128 0x4 /* (abbrev code) */
+ .uleb128 0x24 /* (TAG: DW_TAG_base_type) */
+ .byte 0 /* DW_children_no */
+ .uleb128 0xb /* (DW_AT_byte_size) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3e /* (DW_AT_encoding) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3 /* (DW_AT_name) */
+ .uleb128 0x8 /* (DW_FORM_string) */
+ .byte 0
+ .byte 0
+ .uleb128 0x5 /* (abbrev code) */
+ .uleb128 0x11 /* (TAG: DW_TAG_compile_unit) */
+ .byte 0x1 /* DW_children_yes */
+ .uleb128 0x25 /* (DW_AT_producer) */
+ .uleb128 0xe /* (DW_FORM_strp) */
+ .uleb128 0x13 /* (DW_AT_language) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3 /* (DW_AT_name) */
+ .uleb128 0xe /* (DW_FORM_strp) */
+ .uleb128 0x1b /* (DW_AT_comp_dir) */
+ .uleb128 0xe /* (DW_FORM_strp) */
+ .uleb128 0x11 /* (DW_AT_low_pc) */
+ .uleb128 0x1 /* (DW_FORM_addr) */
+ .uleb128 0x12 /* (DW_AT_high_pc) */
+ .uleb128 0x1 /* (DW_FORM_addr) */
+ .uleb128 0x10 /* (DW_AT_stmt_list) */
+ .uleb128 0x17 /* (DW_FORM_sec_offset) */
+ .byte 0
+ .byte 0
+ .uleb128 0x6 /* (abbrev code) */
+ .uleb128 0x2e /* (TAG: DW_TAG_subprogram) */
+ .byte 0 /* DW_children_no */
+ .uleb128 0x3f /* (DW_AT_external) */
+ .uleb128 0x19 /* (DW_FORM_flag_present) */
+ .uleb128 0x3 /* (DW_AT_name) */
+ .uleb128 0xe /* (DW_FORM_strp) */
+ .uleb128 0x3a /* (DW_AT_decl_file) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3b /* (DW_AT_decl_line) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x49 /* (DW_AT_type) */
+ .uleb128 0x13 /* (DW_FORM_ref4) */
+ .uleb128 0x11 /* (DW_AT_low_pc) */
+ .uleb128 0x1 /* (DW_FORM_addr) */
+ .uleb128 0x12 /* (DW_AT_high_pc) */
+ .uleb128 0x1 /* (DW_FORM_addr) */
+ .uleb128 0x40 /* (DW_AT_frame_base) */
+ .uleb128 0x18 /* (DW_FORM_exprloc) */
+ .uleb128 0x2117 /* (DW_AT_GNU_all_call_sites) */
+ .uleb128 0x19 /* (DW_FORM_flag_present) */
+ .byte 0
+ .byte 0
+ .uleb128 0x7 /* (abbrev code) */
+ .uleb128 0x34 /* (TAG: DW_TAG_variable) */
+ .byte 0 /* DW_children_no */
+ .uleb128 0x3 /* (DW_AT_name) */
+ .uleb128 0x8 /* (DW_FORM_string) */
+ .uleb128 0x3a /* (DW_AT_decl_file) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x3b /* (DW_AT_decl_line) */
+ .uleb128 0xb /* (DW_FORM_data1) */
+ .uleb128 0x49 /* (DW_AT_type) */
+ .uleb128 0x20 /* (DW_FORM_ref_sig8) */
+ .uleb128 0x3f /* (DW_AT_external) */
+ .uleb128 0x19 /* (DW_FORM_flag_present) */
+ .uleb128 0x2 /* (DW_AT_location) */
+ .uleb128 0x18 /* (DW_FORM_exprloc) */
+ .byte 0
+ .byte 0
+
+ /* Manually inserted. */
+ .uleb128 0x8 /* abbrev code */
+ .uleb128 0x2 /* DW_TAG_class_type */
+ .byte 0x0 /* DW_has_children_no */
+ .uleb128 0x47 /* DW_AT_specification */
+ .uleb128 0x13 /* DW_FORM_ref4 */
+ .byte 0x0 /* Terminator */
+ .byte 0x0 /* Terminator */
+ /* End of manual insertion. */
+
+ .byte 0
+
+ .section .debug_aranges,"",@progbits
+ .4byte 0x1c /* Length of Address Ranges Info */
+ .2byte 0x2 /* DWARF Version */
+ .4byte .Ldebug_info0 /* Offset of Compilation Unit Info */
+ .byte 0x4 /* Size of Address */
+ .byte 0 /* Size of Segment Descriptor */
+ .2byte 0 /* Pad to 8 byte boundary */
+ .2byte 0
+ .4byte .Ltext0 /* Address */
+ .4byte .Letext0-.Ltext0 /* Length */
+ .4byte 0
+ .4byte 0
+
+ .section .debug_line,"",@progbits
+.Ldebug_line0:
+
+ .section .debug_str,"MS",@progbits,1
+.LASF0:
+ .string "GNU C++ 4.6.3 20120306"
+.LASF2:
+ .string "."
+.LASF1:
+ .string "pr13961.cc"
+.LASF3:
+ .string "main"
+
diff --git a/gdb/testsuite/gdb.dwarf2/pr13961.exp b/gdb/testsuite/gdb.dwarf2/pr13961.exp
new file mode 100644
index 0000000..c70f2b2
--- /dev/null
+++ b/gdb/testsuite/gdb.dwarf2/pr13961.exp
@@ -0,0 +1,36 @@
+# Copyright 2012 Free Software Foundation, Inc.
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+# A testcase to trigger the code path in bug 13961.
+
+load_lib dwarf.exp
+
+# This test can only be run on targets which support DWARF-2 and use gas.
+if {![dwarf2_support]} {
+ return 0
+}
+
+set testfile "pr13961"
+set srcfile ${testfile}.S
+set binfile ${testfile}.x
+
+if { [prepare_for_testing ${testfile}.exp ${binfile} ${srcfile}] == -1 } {
+ return -1
+}
+
+gdb_test "break main" "Breakpoint.*at.*"
+
+# If we get this far gdb didn't crash.
+pass $testfile
diff --git a/gdb/dwarf2read.c b/gdb/dwarf2read.c
index e002c22..0caa5f9 100644
--- a/gdb/dwarf2read.c
+++ b/gdb/dwarf2read.c
@@ -597,7 +597,7 @@ struct die_reader_specs
gdb_byte *buffer;
};
/* Type of function passed to init_cu_and_read_dies, et.al. */
typedef void (die_reader_func_ftype) (const struct die_reader_specs *reader,
gdb_byte *info_ptr,
struct die_info *comp_unit_die,
@@ -1401,8 +1401,8 @@ static gdb_byte *read_and_check_comp_unit_head
struct dwarf2_section_info *section, gdb_byte *info_ptr,
int is_debug_types_section);
-static void init_tu_and_read_dies
- (struct dwarf2_per_cu_data *this_cu, int keep,
+static void init_cutu_and_read_dies
+ (struct dwarf2_per_cu_data *this_cu, int use_existing_cu, int keep,
die_reader_func_ftype *die_reader_func, void *data);
static void init_cutu_and_read_dies_simple
@@ -2474,7 +2474,7 @@ dw2_get_file_names (struct objfile *objfile,
However, that's not the case for TUs where DW_AT_stmt_list lives in the
DWO file. */
if (this_cu->is_debug_types)
- init_tu_and_read_dies (this_cu, 0, dw2_get_file_names_reader, NULL);
+ init_cutu_and_read_dies (this_cu, 0, 0, dw2_get_file_names_reader, NULL);
else
init_cutu_and_read_dies_simple (this_cu, dw2_get_file_names_reader, NULL);
@@ -3628,8 +3628,7 @@ dwarf2_find_base_address (struct die_info *die, struct dwarf2_cu *cu)
}
}
-/* Subroutine of init_{cu,tu}_and_read_dies.
- Do all the work necessary to initialize THIS_CU->cu and read in its DIE(s).
+/* Initialize a CU (or TU) and read its DIEs.
If the CU defers to a DWO file, read the DWO file as well.
If USE_EXISTING_CU is non-zero, and THIS_CU->cu is non-NULL, then use it.
@@ -3639,18 +3638,13 @@ dwarf2_find_base_address (struct die_info *die, struct dwarf2_cu *cu)
read_in_chain. Otherwise the dwarf2_cu data is freed at the end.
WARNING: If THIS_CU is a "dummy CU" (used as filler by the incremental
- linker) then DIE_READER_FUNC will not get called.
-
- FIXME: As an implementation detail between our callers and us,
- USE_EXISTING_CU and KEEP are OK. But bubbling them up into their callers
- isn't as clean as I'd like. Having more callers with good names
- may be the way to go. */
+ linker) then DIE_READER_FUNC will not get called. */
static void
-init_and_read_dies_worker (struct dwarf2_per_cu_data *this_cu,
- int use_existing_cu, int keep,
- die_reader_func_ftype *die_reader_func,
- void *data)
+init_cutu_and_read_dies (struct dwarf2_per_cu_data *this_cu,
+ int use_existing_cu, int keep,
+ die_reader_func_ftype *die_reader_func,
+ void *data)
{
struct objfile *objfile = dwarf2_per_objfile->objfile;
struct dwarf2_section_info *section = this_cu->info_or_types_section;
@@ -3666,8 +3660,6 @@ init_and_read_dies_worker (struct dwarf2_per_cu_data *this_cu,
if (use_existing_cu)
gdb_assert (keep);
- if (this_cu->is_debug_types)
- gdb_assert (! use_existing_cu);
cleanups = make_cleanup (null_cleanup, NULL);
@@ -3918,43 +3910,6 @@ init_and_read_dies_worker (struct dwarf2_per_cu_data *this_cu,
do_cleanups (cleanups);
}
-/* Main entry point for reading a CU.
- Do all the work necessary to initialize THIS_CU->cu and read in its DIE(s).
- If the CU defers to a DWO file, read the DWO file as well.
-
- If USE_EXISTING_CU is non-zero, and THIS_CU->cu is non-NULL, then use it.
- Otherwise, a new CU is allocated with xmalloc.
-
- If KEEP is non-zero, then if we allocated a dwarf2_cu we add it to
- read_in_chain. Otherwise the dwarf2_cu data is freed at the end. */
-
-static void
-init_cu_and_read_dies (struct dwarf2_per_cu_data *this_cu,
- int use_existing_cu, int keep,
- die_reader_func_ftype *die_reader_func,
- void *data)
-{
- init_and_read_dies_worker (this_cu, use_existing_cu, keep,
- die_reader_func, data);
-}
-
-/* Main entry point for reading a TU.
- Do all the work necessary to initialize THIS_CU->cu and read in its DIE(s).
- If the TU defers to a DWO file, read the DWO file as well.
-
- If KEEP is non-zero, then if we allocated a dwarf2_cu we add it to
- read_in_chain. Otherwise the dwarf2_cu data is freed at the end. */
-
-static void
-init_tu_and_read_dies (struct dwarf2_per_cu_data *this_cu,
- int keep,
- die_reader_func_ftype *die_reader_func,
- void *data)
-{
- gdb_assert (this_cu->is_debug_types);
- init_and_read_dies_worker (this_cu, 0, keep, die_reader_func, data);
-}
-
/* Read CU/TU THIS_CU in section SECTION,
but do not follow DW_AT_GNU_dwo_name if present.
DWO_FILE, if non-NULL, is the DWO file to read (the caller is assumed to
@@ -4176,8 +4131,8 @@ process_psymtab_comp_unit (struct dwarf2_per_cu_data *this_cu)
free_one_cached_comp_unit (this_cu);
gdb_assert (! this_cu->is_debug_types);
- init_cu_and_read_dies (this_cu, 0, 0, process_psymtab_comp_unit_reader,
- NULL);
+ init_cutu_and_read_dies (this_cu, 0, 0, process_psymtab_comp_unit_reader,
+ NULL);
/* Age out any secondary CUs. */
age_cached_comp_units ();
@@ -4192,6 +4147,7 @@ process_psymtab_type_unit (void **slot, void *info)
struct signatured_type *sig_type = (struct signatured_type *) *slot;
struct dwarf2_per_cu_data *per_cu = &sig_type->per_cu;
+ gdb_assert (per_cu->is_debug_types);
gdb_assert (info == NULL);
/* If this compilation unit was already read in, free the
@@ -4202,8 +4158,8 @@ process_psymtab_type_unit (void **slot, void *info)
if (per_cu->cu != NULL)
free_one_cached_comp_unit (per_cu);
- gdb_assert (per_cu->is_debug_types);
- init_tu_and_read_dies (per_cu, 0, process_psymtab_comp_unit_reader, NULL);
+ init_cutu_and_read_dies (per_cu, 0, 0, process_psymtab_comp_unit_reader,
+ NULL);
/* Age out any secondary CUs. */
age_cached_comp_units ();
@@ -4303,10 +4259,7 @@ load_partial_comp_unit_reader (const struct die_reader_specs *reader,
static void
load_partial_comp_unit (struct dwarf2_per_cu_data *this_cu)
{
- if (this_cu->is_debug_types)
- init_tu_and_read_dies (this_cu, 1, load_partial_comp_unit_reader, NULL);
- else
- init_cu_and_read_dies (this_cu, 0, 1, load_partial_comp_unit_reader, NULL);
+ init_cutu_and_read_dies (this_cu, 1, 1, load_partial_comp_unit_reader, NULL);
}
/* Create a list of all compilation units in OBJFILE.
@@ -5315,7 +5268,7 @@ load_full_comp_unit (struct dwarf2_per_cu_data *this_cu)
{
gdb_assert (! this_cu->is_debug_types);
- init_cu_and_read_dies (this_cu, 1, 1, load_full_comp_unit_reader, NULL);
+ init_cutu_and_read_dies (this_cu, 1, 1, load_full_comp_unit_reader, NULL);
}
/* Add a DIE to the delayed physname list. */
@@ -11176,11 +11129,14 @@ find_partial_die (sect_offset offset, struct dwarf2_cu *cu)
if (pd == NULL && per_cu->load_all_dies == 0)
{
- /* FIXME: The testsuite doesn't trigger this code path.
- http://sourceware.org/bugzilla/show_bug.cgi?id=13961 */
-
- free_one_cached_comp_unit (per_cu);
per_cu->load_all_dies = 1;
+
+ /* This is nasty. When we reread the DIEs, somewhere up the call chain
+ THIS_CU->cu may already be in use. So we can't just free it and
+ replace its DIEs with the ones we read in. Instead, we leave those
+ DIEs alone (which can still be in use, e.g. in scan_partial_symbols),
+ and clobber THIS_CU->cu->partial_dies with the hash table for the new
+ set. */
load_partial_comp_unit (per_cu);
pd = find_partial_die_in_comp_unit (offset, per_cu->cu);
@@ -15913,7 +15869,7 @@ read_signatured_type (struct signatured_type *sig_type)
gdb_assert (per_cu->is_debug_types);
gdb_assert (per_cu->cu == NULL);
- init_tu_and_read_dies (per_cu, 1, read_signatured_type_reader, NULL);
+ init_cutu_and_read_dies (per_cu, 0, 1, read_signatured_type_reader, NULL);
}
/* Decode simple location descriptions.
@@ -597,7 +597,7 @@ struct die_reader_specs
gdb_byte *buffer;
};
-/* Type of function passed to init_cu_and_read_dies, et.al. */
+/* Type of function passed to init_cutu_and_read_dies, et.al. */
typedef void (die_reader_func_ftype) (const struct die_reader_specs *reader,
gdb_byte *info_ptr,